@@ -99,6 +99,13 @@ page_view_set_selected (PageView *view, gboolean selected)
 }
 
 
+gboolean page_view_get_selected (PageView *view)
+{
+    g_return_val_if_fail (view != NULL, FALSE);
+    return view->priv->selected;
+}
+
+
 void
 page_view_set_x_offset (PageView *view, gint offset)
 {
@@ -157,16 +164,16 @@ set_pixel (guchar *input, gint rowstride, gint n_channels,
      *      l  L           R   r 
      *   +-----+-----+-----+-----+
      *   |     |     |     |     |
-     * t |  +--|-----|-----|---+ |
-     * T +-----+-----+-----+-----+
+     * t |  +--+-----+-----+---+ |
+     * T +--+--+-----+-----+---+-+
      *   |  |  |     |     |   | |
      *   |  |  |     |     |   | |
-     *   +-----+-----+-----+-----+
+     *   +--+--+-----+-----+---+-+
      *   |  |  |     |     |   | |
      *   |  |  |     |     |   | |
-     * B +-----+-----+-----+-----+
+     * B +--+--+-----+-----+---+-+
      *   |  |  |     |     |   | |
-     * b |  +--|-----|-----|---+ |
+     * b |  +--+-----+-----+---+ |
      *   +-----+-----+-----+-----+
      * 
      * 
@@ -177,10 +184,10 @@ set_pixel (guchar *input, gint rowstride, gint n_channels,
      *   |     |     |     |     |
      *   |     |     |     |     |
      *   +-----+-----+-----+-----+
-     * t |     |   +-|--+  |     |
+     * t |     |   +-+--+  |     |
      *   |     |   | |  |  |     |
-     *   +-----+-----+-----+-----+
-     * b |     |   +-|--+  |     |
+     *   +-----+---+-+--+--+-----+
+     * b |     |   +-+--+  |     |
      *   |     |     |     |     |
      *   +-----+-----+-----+-----+
      *   |     |     |     |     |
@@ -190,27 +197,30 @@ set_pixel (guchar *input, gint rowstride, gint n_channels,
      * Same again, just no completely covered pixels.
      */
   
-    L = (gint)(l + 1.0);
-    R = (gint)(r);
-    T = (gint)(t + 1.0);
-    B = (gint)(b);
+    L = l;
+    if (L != l)
+        L++;
+    R = r;
+    T = t;
+    if (T != t)
+        T++;
+    B = b;
   
     red = green = blue = 0.0;
 
     /* Target can fit inside one source pixel
      * +-----+
      * |     |
-     * | +--+|      +-----+      +-----+-----+
-     * +-----+  or  | +-+ |  or  |   +-|+    |
-     * | +--+|      | +-+ |      |   +-|+    |
-     * |     |      +-----+      +-----+-----+
+     * | +--+|      +-----+-----+      +-----+      +-----+      +-----+
+     * +-+--++  or  |   +-++    |  or  | +-+ |  or  | +--+|  or  |  +--+
+     * | +--+|      |   +-++    |      | +-+ |      | |  ||      |  |  |
+     * |     |      +-----+-----+      +-----+      +-+--++      +--+--+
      * +-----+
      */
-    /* FIXME: This only handles the square case - pixels could be stretched in the general case */
-    if (r - l < 1.0 || b - t < 1.0) {
+    if ((r - l <= 1.0 && (gint)r == (gint)l) || (b - t <= 1.0 && (gint)b == (gint)t)) {
         /* Inside */
-        if (L > R && T > B) {
-            guchar *p = get_pixel (input, rowstride, n_channels, R, B);
+        if ((gint)l == (gint)r || (gint)t == (gint)b) {
+            guchar *p = get_pixel (input, rowstride, n_channels, (gint)l, (gint)t);
             pixel[0] = p[0];
             pixel[1] = p[1];
             pixel[2] = p[2];
@@ -408,10 +418,10 @@ update_preview (GdkPixbuf *image,
 
     /* FIXME: There's an off by one error in there somewhere... */
     if (R >= output_width)
-      R = output_width - 1;
+        R = output_width - 1;
     if (B >= output_height)
-      B = output_height - 1;
-  
+        B = output_height - 1;
+
     g_return_if_fail (L >= 0);
     g_return_if_fail (R < output_width);
     g_return_if_fail (T >= 0);
@@ -446,18 +456,20 @@ static void
 update_page_view (PageView *view)
 {
     GdkPixbuf *image;
-    gint scan_line;
+    gint old_scan_line, scan_line;
 
     if (!view->priv->update_image)
         return;
 
     image = page_get_image (view->priv->page);
+    old_scan_line = view->priv->scan_line;
     scan_line = page_get_scan_line (view->priv->page);
+
     update_preview (image,
                     &view->priv->image,
                     view->priv->width - view->priv->border_width * 2,
                     view->priv->height - view->priv->border_width * 2,
-                    page_get_orientation (view->priv->page), view->priv->scan_line, scan_line);
+                    page_get_orientation (view->priv->page), old_scan_line, scan_line);
     g_object_unref (image);
 
     view->priv->update_image = FALSE;
@@ -523,7 +535,11 @@ get_crop_location (PageView *view, gint x, gint y)
         return CROP_MIDDLE;
     }
 
-    // FIXME: Adjust edges when small
+    /* Adjust borders so can select */
+    if (dw < crop_border * 3)
+        crop_border = dw / 3;
+    if (dh < crop_border * 3)
+        crop_border = dh / 3;
 
     /* Top left */
     if (ix < crop_border && iy < crop_border)
@@ -586,6 +602,9 @@ page_view_motion (PageView *view, gint x, gint y)
     gint new_x, new_y, new_w, new_h;
     CropLocation location;
     gint cursor;
+    gint min_size;
+  
+    min_size = screen_to_page_x (view, 15);
 
     g_return_if_fail (view != NULL);
   
@@ -641,19 +660,20 @@ page_view_motion (PageView *view, gint x, gint y)
     new_w = view->priv->selected_crop_w;
     new_h = view->priv->selected_crop_h;
 
+    /* Limit motion to remain within page and minimum crop size */
     if (view->priv->crop_location == CROP_TOP_LEFT ||
         view->priv->crop_location == CROP_LEFT ||
         view->priv->crop_location == CROP_BOTTOM_LEFT) {
-        if (dx > new_w + 1)
-            dx = new_w + 1;
+        if (dx > new_w - min_size)
+            dx = new_w - min_size;
         if (new_x + dx < 0)
             dx = -new_x;
     }
     if (view->priv->crop_location == CROP_TOP_LEFT ||
         view->priv->crop_location == CROP_TOP ||
         view->priv->crop_location == CROP_TOP_RIGHT) {
-        if (dy > new_h + 1)
-            dy = new_h + 1;
+        if (dy > new_h - min_size)
+            dy = new_h - min_size;
         if (new_y + dy < 0)
             dy = -new_y;
     }
@@ -661,20 +681,31 @@ page_view_motion (PageView *view, gint x, gint y)
     if (view->priv->crop_location == CROP_TOP_RIGHT ||
         view->priv->crop_location == CROP_RIGHT ||
         view->priv->crop_location == CROP_BOTTOM_RIGHT) {
-        if (new_w - dx < 1)
-            dx = new_w - 1;
+        if (dx < min_size - new_w)
+            dx = min_size - new_w;
         if (new_x + new_w + dx > pw)
             dx = pw - new_x - new_w;
     }
     if (view->priv->crop_location == CROP_BOTTOM_LEFT ||
         view->priv->crop_location == CROP_BOTTOM ||
         view->priv->crop_location == CROP_BOTTOM_RIGHT) {
-        if (new_h - dy < 1)
-            dy = new_h - 1;
+        if (dy < min_size - new_h)
+            dy = min_size - new_h;
         if (new_y + new_h + dy > ph)
             dy = ph - new_y - new_h;
     }
+    if (view->priv->crop_location == CROP_MIDDLE) {
+        if (new_x + dx + new_w > pw)
+            dx = pw - new_x - new_w;
+        if (new_x + dx < 0)
+            dx = -new_x;
+        if (new_y + dy + new_h > ph)
+            dy = ph - new_y - new_h;
+        if (new_y + dy  < 0)
+            dy = -new_y;
+    }
 
+    /* Move crop */
     if (view->priv->crop_location == CROP_MIDDLE) {
         new_x += dx;
         new_y += dy;          
@@ -704,21 +735,9 @@ page_view_motion (PageView *view, gint x, gint y)
         new_h += dy;
     }
 
-    if (new_w < 1)
-        new_w = 1;
-    if (new_h < 1)
-        new_h = 1;
-
-    if (new_x > pw - cw)
-        new_x = pw - cw;
-    if (new_x < 0)
-        new_x = 0;
-    if (new_y > ph - ch)
-        new_y = ph - ch;
-    if (new_y < 0)
-        new_y = 0;
-
     page_move_crop (view->priv->page, new_x, new_y);
+
+    /* If reshaped crop, must be a custom crop */
     if (new_w != cw || new_h != ch)
         page_set_custom_crop (view->priv->page, new_w, new_h);
 }
@@ -756,8 +775,8 @@ static void
 update_animation (PageView *view)
 {
     gboolean animate, is_animating;
-  
-    animate = page_get_scan_line (view->priv->page) == 0;
+
+    animate = page_is_scanning (view->priv->page) && !page_has_data (view->priv->page);
     is_animating = view->priv->animate_timeout != 0;
     if (animate == is_animating)
         return;
@@ -779,7 +798,6 @@ update_animation (PageView *view)
 void
 page_view_render (PageView *view, cairo_t *context)
 {
-    gint scan_line;
     gint width, height;
 
     g_return_if_fail (view != NULL);
@@ -795,10 +813,7 @@ page_view_render (PageView *view, cairo_t *context)
     cairo_translate (context, view->priv->x, view->priv->y);
 
     /* Draw page border */
-    if (view->priv->selected)
-        cairo_set_source_rgb (context, 1, 0, 0);
-    else
-        cairo_set_source_rgb (context, 0, 0, 0);
+    cairo_set_source_rgb (context, 0, 0, 0);
     cairo_set_line_width (context, view->priv->border_width);
     cairo_rectangle (context,
                      (double)view->priv->border_width / 2,
@@ -813,8 +828,7 @@ page_view_render (PageView *view, cairo_t *context)
     cairo_paint (context);
 
     /* Draw throbber */
-    scan_line = page_get_scan_line (view->priv->page);
-    if (scan_line == 0) {
+    if (page_is_scanning (view->priv->page) && !page_has_data (view->priv->page)) {
         gdouble inner_radius, outer_radius, x, y, arc, offset = 0.0;
         gint i;
 
@@ -845,9 +859,12 @@ page_view_render (PageView *view, cairo_t *context)
     }
 
     /* Draw scan line */
-    if (scan_line > 0) {
+    if (page_is_scanning (view->priv->page) && page_get_scan_line (view->priv->page) > 0) {
+        gint scan_line;
         double s;
         double x1, y1, x2, y2;
+
+        scan_line = page_get_scan_line (view->priv->page);
         
         switch (page_get_orientation (view->priv->page)) {
         case TOP_TO_BOTTOM:
@@ -900,7 +917,7 @@ page_view_render (PageView *view, cairo_t *context)
         cairo_new_sub_path (context);
         cairo_rectangle (context, dx, dy, dw, dh);
         cairo_set_fill_rule (context, CAIRO_FILL_RULE_EVEN_ODD);
-        cairo_set_source_rgba (context, 0.5, 0.5, 0.5, 0.5);
+        cairo_set_source_rgba (context, 0.25, 0.25, 0.25, 0.1);
         cairo_fill (context);
         
         /* Show new edge */
