@@ -188,7 +188,7 @@ scanner_scanning_changed_cb (Scanner *scanner)
 
 
 static void
-scan_cb (SimpleScan *ui, const gchar *device, gint dpi, const gchar *profile_name, gboolean continuous)
+scan_cb (SimpleScan *ui, const gchar *device, gint dpi, const gchar *profile_name, ScanType type)
 {
     /* Default filename to use when saving document (and extension will be added, e.g. .jpg) */
     const gchar *filename_prefix = _("Scanned Document");
@@ -216,7 +216,7 @@ scan_cb (SimpleScan *ui, const gchar *device, gint dpi, const gchar *profile_nam
     filename = g_strdup_printf ("%s.%s", filename_prefix, profiles[i].extension);
     ui_set_default_file_name (ui, filename);
     g_free (filename);
-    scanner_scan (scanner, device, dpi, profiles[i].mode, 8, continuous);
+    scanner_scan (scanner, device, dpi, profiles[i].mode, 8, type);
 }
 
 
@@ -240,6 +240,8 @@ save_book (const gchar *uri, GError **error)
         result = book_save_ps (book, uri, error);
     else if (g_str_has_suffix (uri_lower, ".png"))
         result = book_save_png (book, uri, error);
+    else if (g_str_has_suffix (uri_lower, ".tif") || g_str_has_suffix (uri_lower, ".tiff"))
+        result = book_save_tiff (book, uri, error);
     else
         result = book_save_jpeg (book, uri, error);
 
@@ -269,7 +271,7 @@ save_cb (SimpleScan *ui, const gchar *uri)
 
 
 static gchar *
-get_temporary_filename (const gchar *extension)
+get_temporary_filename (const gchar *prefix, const gchar *extension)
 {
     gint fd;
     gchar *filename, *path;
@@ -278,7 +280,7 @@ get_temporary_filename (const gchar *extension)
     /* NOTE: I'm not sure if this is a 100% safe strategy to use g_file_open_tmp(), close and
      * use the filename but it appears to work in practise */
 
-    filename = g_strdup_printf ("scanned-document-XXXXXX.%s", extension);
+    filename = g_strdup_printf ("%s-XXXXXX.%s", prefix, extension);
     fd = g_file_open_tmp (filename, &path, &error);
     g_free (filename);
     if (fd < 0) {
@@ -306,7 +308,7 @@ email_cb (SimpleScan *ui, const gchar *profile)
         gchar *path, *uri;
 
         /* Open a temporary file */
-        path = get_temporary_filename ("pdf");
+        path = get_temporary_filename ("scanned-document", "pdf");
         if (path) {
             uri = g_filename_to_uri (path, NULL, NULL);
             saved = book_save_pdf (book, uri, &error);
@@ -321,7 +323,7 @@ email_cb (SimpleScan *ui, const gchar *profile)
         for (i = 0; i < book_get_n_pages (book); i++) {
             gchar *path, *uri;
 
-            path = get_temporary_filename ("jpeg");
+            path = get_temporary_filename ("scanned-document", "jpg");
             if (!path) {
                 saved = FALSE;
                 break;
